import {
  Metadata, Http, IDictionary, Router,
} from '@zeedhi/core';
import {
  Col, Text, Textarea, Card, Checkbox, Button, Modal, ModalService, LoadingService, AlertService, DialogService, FileInput,
} from '@zeedhi/common';
import { IterableComponentRender, IterableCarousel } from '@zeedhi/teknisa-components-common';
import { getRouteParams } from '../router';
import FileUpload from './FileUpload';

export class DetalhamentoTarefaController {
  private anexoModal!: Modal;

  private fileInputModal!: Modal;

  private ImagesModal!: Modal;

  // Controla a visibilidade dos elementos, referente ao nível de acesso do usuário
  public isDisabledElement = (localStorage.getItem('idpermissstarefas') === 'V');

  public changeButton() {
    const buttonStart = Metadata.getInstance<Col>('colButtonStartTask');
    const buttonPress = Metadata.getInstance<Col>('colButtonPress');
    buttonStart.isVisible = !buttonStart.isVisible;
    buttonPress.isVisible = !buttonPress.isVisible;
  }

  public addTextArea({ component }:{ component:Button }) {
    const buttonBusy = Metadata.getInstance<Button>('buttonBusy');
    const buttonWtAcess = Metadata.getInstance<Button>('buttonWtAcess');
    const buttonCleanPlace = Metadata.getInstance<Button>('buttonCleanPlace');
    const buttonOtherReasons = Metadata.getInstance<Button>('buttonOtherReasons');
    buttonBusy.outline = !(component.label === buttonBusy.label);
    buttonWtAcess.outline = !(component.label === buttonWtAcess.label);
    buttonCleanPlace.outline = !(component.label === buttonCleanPlace.label);
    buttonOtherReasons.outline = !(component.label === buttonOtherReasons.label);
    const textA = Metadata.getInstance<Textarea>('textarea');
    if (component.label === buttonOtherReasons.label) {
      textA.isVisible = true;
    } else {
      textA.isVisible = false;
    }
  }

  public async statusCheck() {
    let chkStatus: boolean = false;
    await Http.post('/statusCheck', { NRFUNCIONARIO: localStorage.getItem('UserLogado') }).then((response: IDictionary) => {
      chkStatus = response.data.dataset.statuschk[0]['COUNT(IDSITUACAOAGEND)'];
    });
    return chkStatus;
  }

  public showDetailsTask() {
    const routeParams = getRouteParams();
    const { nr } = routeParams;
    Http.post('/showDetailsTask', { nr }).then((response: IDictionary) => {
      response.data.dataset.cardDetails.forEach((dados: IDictionary) => {
        const time = dados.DTHORAAGEND.split(' ')[1];
        Metadata.getInstance<Text>('statusTask').text = this.stylizeStatus(dados.IDSITUACAOAGEND);
        Metadata.getInstance<Text>('nameTask').text = dados.NMCHECKLISTAGEND;
        Metadata.getInstance<Text>('valuetime').text = time;
        Metadata.getInstance<Text>('namePlace').text = dados.NMLOCALCLIENTE;
        Metadata.getInstance<Text>('nameTypePlace').text = dados.NMLOCALPAI;
        new Promise((resolve, reject) => {
          try {
            resolve(this.statusCheck());
          } catch (error) {
            reject(error);
          }
        }).then((chk: any) => {
          if (dados.IDSITUACAOAGEND === 'Em andamento') {
            this.changeButton();
          } else if (chk) {
            Metadata.getInstance<Card>('cardButtons').isVisible = false;
          }
        });
      });
    });
  }

  public itensDetailsTask() {
    const routeParams = getRouteParams();
    const { nr } = routeParams;
    Http.post('/getItensDetailsTask', { nr }).then((response: IDictionary) => {
      response.data.dataset.cardItensDetails.forEach((dados: IDictionary) => {
        Metadata.getInstance<IterableComponentRender>('iCardDetailsTask').datasource.data.push({
          NRTAREFA: dados.NRTAREFAAGEND,
          NMTAREFA: dados.NMTAREFA,
        });
      });
    });
  }

  public stylizeStatus(status: string) {
    if (status === 'Pendente') {
      return "<span  style='background-color:#dee2e6; color: #ff6700; font-size: 10px;'>P E N D E N T E</span>";
    }
    if (status === 'Em andamento') {
      return "<span  style='background-color: #dee2e6; color: #023e8a; font-size: 11px;'>EM ANDAMENTO</span>";
    }
    if (status === 'Atrasado') {
      return "<span  style='background-color: #dee2e6; color: red; font-size: 10px;'>A T R A S A D O</span>";
    }
    return '';
  }

  public startTask() { // Iniciar tarefa na tela de detalhamento
    const routeParams = getRouteParams();
    const { nr } = routeParams;
    Http.post('/changeStatusTask', {
      button: 2,
      NRTAREFAATIVA: nr,
    }).then(() => {
      this.changeButton();
      Metadata.getInstance<Text>('statusTask').text = "<span  style='background-color: #dee2e6; color: #023e8a; font-size: 11px;'>EM ANDAMENTO</span>";
    });
  }

  public finishTask() { // Finalizar tarefa pela tela de detalhamento
    const { nr } = getRouteParams();
    const chkbox: IDictionary = [];
    Metadata.getInstance<IterableComponentRender>('iCardDetailsTask').datasource.data.forEach((dataBase: IDictionary) => {
      if (Metadata.getInstance<Checkbox>(`checkbox_${dataBase.NRTAREFA}`).value) {
        chkbox.push(dataBase.NRTAREFA);
      }
    });
    Http.post('/finishTask', {
      NRTAREFAATIVA: nr,
      CHKBOX: chkbox,
      DSOBSTAREFAAGEND: Metadata.getInstance<Textarea>('box-observation').value,
    }).then(() => {
      Router.replace('/Tarefas');
    });
  }

  public setRoute({ component }:{ component: Button }) {
    const { nr } = getRouteParams();
    component.to = `/Reportar_Tarefa/${nr}`;
  }

  public async creatanexoModal() {
    this.anexoModal = await ModalService.create({
      name: 'anexoModal',
      children: [
        {
          name: 'fromTypescriptHeader',
          component: 'ZdHeader',
          color: 'transparent',
          padless: true,
          elevation: 0,
          leftSlot: [
            {
              name: 'titleText',
              component: 'ZdText',
              text: 'Oque você deseja fazer?',
              cssClass: 'corPadrao',
            },
          ],
          rightSlot: [
            {
              name: 'closeModalButton',
              component: 'ZdButton',
              iconName: 'mdi-close',
              icon: true,
              events: {
                click: '{{DetalhamentoTarefaController.closanexoModal}}',
              },
            },
          ],
        },
        {
          name: 'form-anexo',
          component: 'ZdForm',
          cssClass: 'formAnexo',
          children: [
            {
              name: 'Btn-anexar-AnexoModal',
              label: 'Anexar Imagens',
              component: 'ZdButton',
              cssClass: 'btnCamera',
              events: {
                click: '{{DetalhamentoTarefaController.creatfileInputModal}}',
              },
            },
            {
              name: 'Btn-visualizarAnexo-AnexoModal',
              label: 'Exibir Imagens anexadas',
              outline: true,
              component: 'ZdButton',
              cssClass: 'btnCamera',
              events: {
                click: '{{DetalhamentoTarefaController.creatImagesModal}}',
              },
            },
          ],
        },
      ],
    });
    this.anexoModal.show();
  }

  public async closanexoModal() {
    await this.anexoModal.hide();
    this.anexoModal?.destroy();
  }

  public async creatfileInputModal() {
    this.fileInputModal = await ModalService.create({
      name: 'fileInputModal',
      fullscreen: true,
      children: [
        {
          name: 'HeaderFileInputModal',
          component: 'ZdHeader',
          color: 'transparent',
          padless: true,
          elevation: 0,
          leftSlot: [
            {
              name: 'titleText',
              component: 'ZdText',
              text: 'Anexar imagens',
              cssClass: 'corPadrao',
              cssStyle: {
                'font-size': '1.5em',
              },
            },
          ],
          rightSlot: [
            {
              name: 'closeModalButton',
              component: 'ZdButton',
              iconName: 'mdi-close',
              icon: true,
              events: {
                click: '{{DetalhamentoTarefaController.closfileInputModal}}',
              },
            },
          ],
        },
        {
          name: 'form-anexo',
          component: 'ZdForm',
          cssClass: 'formAnexo',
          children: [
            {
              name: 'FileInput-ImagensDoAtivo_carouselImagesAnexoTask',
              component: 'ZdFileInput',
              showDragArea: true,
              dragAreaHeight: '300px',
              multiple: true,
              showBorder: false,
              chips: true,
              removableChip: false,
              accept: '.png,.jpg,.jpeg',
              events: {
                change: '{{DetalhamentoTarefaController.exibirImagem}}',
              },
            },
            {
              name: 'LinhaCarrosel',
              component: 'ZdCol',
              children: [
                {
                  name: 'carouselImagesAnexoTask',
                  component: 'TekIterableCarousel',
                  slidesPerView: 1,
                  infiniteScroll: false,
                  iterable: {
                    datasource: {
                      uniqueKey: 'id',
                      data: [],
                    },
                    componentMetadata: {
                      name: 'LinhaContendoImagensAtivos_<<row.id>>',
                      component: 'ZdRow',
                      justify: 'center',
                      children: [
                        {
                          name: 'imagemDoAtivo_<<row.id>>',
                          component: 'ZdImage',
                          src: '<<row.src>>',
                          width: '25vw',
                        },
                      ],
                    },
                  },
                },
              ],
              events: {
                onMounted: '{{DetalhamentoTarefaController.invisivel}}',
              },
            },
            {
              name: 'Btn-fileInputModal',
              label: 'Anexar',
              component: 'ZdButton',
              width: '95%',
              events: {
                click: '{{DetalhamentoTarefaController.anexarImage}}',
              },
            },
          ],
        },
      ],
    });
    this.fileInputModal.show();
  }

  public async closfileInputModal() {
    await this.fileInputModal.hide();
    this.fileInputModal?.destroy();
  }

  public invisivel({ component }: any) {
    component.isVisible = false;
  }

  public async creatImagesModal() {
    /* eslint-disable */
    this.ImagesModal = await ModalService.create({
      name: 'ImagesModal',
      children: [
        {
          name: 'HeaderImagesModal',
          component: 'ZdHeader',
          color: 'transparent',
          padless: true,
          elevation: 0,
          leftSlot: [
            {
              name: 'titleText',
              component: 'ZdText',
              text: 'Imagens anexada',
              cssClass: 'corPadrao',
              cssStyle: {
                'font-size': '1.5em',
              },
            },
          ],
          rightSlot: [
            {
              name: 'closeModalButton',
              component: 'ZdButton',
              iconName: 'mdi-close',
              icon: true,
              events: {
                click: '{{DetalhamentoTarefaController.closImagesModal}}',
              },
            },
          ],
        },
        {
          name: 'form-anexo',
          component: 'ZdForm',
          cssClass: 'formAnexo',
          children: [
            {
              name: 'carouselImagesAnexoTask',
              component: 'TekIterableCarousel',
              center: true,
              slidesPerView: 1,
              infiniteScroll: false,
              events: {
                onCreated: '{{DetalhamentoTarefaController.getImage}}',
                onMounted: ({ element }) => {
                  if (element) {
                    setTimeout(() => { 
                      const hooper: any = element.querySelector('.hooper');
                      const hooperObj = hooper.__vue__;
                      hooperObj.updateWidth();
                    }, 300);              
                  }
                },
              },
              iterable: {
                componentMetadata: {
                  name: 'LinhaContendoImagensAtivos_<<row.id>>',
                  component: 'ZdRow',
                  justify: 'center',
                  children: [
                    {
                      name: `imagemDoAtivo_${(window.location.pathname.split('/')[1] == 'Detalhes_Tarefa') ? '<<row.NRFOTOTAREFAAGEND>>' : '<<row.NRFOTOTAREFAGERCHK>>'}`,
                      component: 'ZdImage',
                      src: (window.location.pathname.split('/')[1] == 'Detalhes_Tarefa') ? '<<row.DSFOTOTAREFAAGEND>>' : '<<row.DSFOTOTAREFAGERCHK>>',
                      width: '35vw',
                      height: '50vw',
                    },
                  ],
                },
              },
            },
          ],
        },
        {
          name: 'Btn-DeleteImage',
          component: 'ZdButton',
          label: 'Apagar imagem',
          width: '95%',
          events:{
            click: '{{DetalhamentoTarefaController.deletImage}}',
          }
        },
      ],
    });
    this.ImagesModal.show();
  }

  public async closImagesModal() {
    await this.ImagesModal.hide();
    this.ImagesModal?.destroy();
  }

  public exibirImagem({ component }: any) {
    const carrosel = Metadata.getInstance<IterableCarousel>(component.name.split('_')[1]);
    const row = Metadata.getInstance<Col>('LinhaCarrosel');

    const imagens: any = [];
    if (component.value.length) {
      row.isVisible = true;
      component.value.forEach((imagem: any, pos: number) => {
        imagens.push({
          id: `cliente_${pos}`,
          src: window.URL.createObjectURL(imagem),
        });
      });
    } else {
      row.isVisible = false;
    }
    carrosel.iterable.datasource.updateData(imagens);
  }

  public async anexarImage() {
    const imagensConvertidas = await FileUpload.converterImagens('FileInput-ImagensDoAtivo_carouselImagesAnexoTask');
    if (imagensConvertidas.length > 0) {
      LoadingService.show('Salvando...');
      const link = (window.location.pathname.split('/')[1] == 'Detalhes_Tarefa') ? '/anexarFotoTarefa' : '/anexarFotoVistoria';
      Http.post(link, {
          IMAGEM: imagensConvertidas,
          CDOPERADOR: window.localStorage.getItem('CDOPERADOR'),
          NRTAREFA: window.location.pathname.split('/')[2],
        }).then((response: any) => {
        LoadingService.hide();
        if('success' === response.data){
          const file = Metadata.getInstance<FileInput>('FileInput-ImagensDoAtivo_carouselImagesAnexoTask');
          file.value = [];
          this.exibirImagem({ component: file });
          this.messages('OK', 'Imagens anexadas com sucesso!')
        } else {
          this.messages('Error', 'Ocorreu um erro ao salvar as imagens.<br>.Tente novamente.<br><br>Se o problema persistir entre em contato com o suporte.')
        }
      });
    }
  }

  public async deletImage(){
    LoadingService.show('Deletando...');
    const carrosel = Metadata.getInstance<IterableCarousel>('carouselImagesAnexoTask');
    let img = carrosel.iterable.datasource.data;
    const [deleteData] = img.filter((value: any, index: any) => { return index === carrosel.currentSlide });
    const link = (window.location.pathname.split('/')[1] == 'Detalhes_Tarefa') ? '/deleteFotoTarefa' : '/deleteFotoVistoria';
    Http.get(`${link}?NRTAREFA=${window.location.pathname.split('/')[2]}&&NRFOTOTAREFA=${deleteData.NRFOTOTAREFAAGEND || deleteData.NRFOTOTAREFAGERCHK}`).then((response: any) => {
      LoadingService.hide();
      if('success' == response.data) {
        const newData = img.filter((value: any, index: any) => { return index !== carrosel.currentSlide });
        carrosel.iterable.datasource.updateData(newData);
      }
    });
  }

  
  public messages(returnMessage: string, mensagem: string) {
    if (returnMessage === 'OK') {
      AlertService.show({
        name: 'basic-alert',
        text: mensagem,
        top: true,
        left: false,
        timeout: 3000,
        color: 'success',
      });
    } else {
      DialogService.show({
        name: 'warning-dialog',
        type: 'warning',
        title: 'Erro!',
        text: mensagem,
      });
    }
  }

  public getImage({ component }: any) {
    LoadingService.show('Abrindo Imagens...');
    const link = (window.location.pathname.split('/')[1] == 'Detalhes_Tarefa') ? 'Tarefa' : 'Vistoria';
    Http.get(`/getFoto${link}?NRTAREFA=${window.location.pathname.split('/')[2]}`).then((response: any) => {
      LoadingService.hide();
      component.iterable.datasource.data = response.data;
      if(response.data.length == 0) {
        Metadata.getInstance<Button>('Btn-DeleteImage').isVisible = false;
      }
    })
  }
}
